@@ -55,7 +55,7 @@ static int raw=0;
 #endif
 static kate_float repeat_threshold=(kate_float)0;
 static kate_float keepalive_threshold=(kate_float)0;
-static kate_float last_stream_time=(kate_float)0;
+static kate_int64_t last_stream_time=0;
 static const unsigned char utf8bom[3]={0xef,0xbb,0xbf};
 static const unsigned char utf16lebom[2]={0xff,0xfe};
 static const unsigned char utf16bebom[2]={0xfe,0xff};
@@ -98,8 +98,20 @@ int write_headers(FILE *f)
   ogg_packet op;
 
   /* command line overrides */
-  if (language) kate_info_set_language(&ki,language);
-  if (category) kate_info_set_category(&ki,category);
+  if (language) {
+    int ret=kate_info_set_language(&ki,language);
+    if (ret<0) {
+      fprintf(stderr,"Failed to set stream language to '%s': %d\n",language,ret);
+      return ret;
+    }
+  }
+  if (category) {
+    int ret=kate_info_set_category(&ki,category);
+    if (ret<0) {
+      fprintf(stderr,"Failed to set stream category to '%s': %d\n",category,ret);
+      return ret;
+    }
+  }
 
   /* warn if either language or category is missing */
   if (!ki.category || !*ki.category) {
@@ -119,7 +131,7 @@ int write_headers(FILE *f)
 
 #ifdef DEBUG
     if (raw) {
-      int ret=send_packet(f,&op,(kate_float)-1);
+      int ret=send_packet(f,&op,-1);
       if (ret<0) {
         fprintf(stderr,"error sending packet: %d\n",ret);
         return ret;
@@ -139,15 +151,14 @@ int write_headers(FILE *f)
   return flush_page(f);
 }
 
-int send_packet(FILE *f,ogg_packet *op,kate_float t)
+int send_packet(FILE *f,ogg_packet *op,kate_int64_t t)
 {
-  int ret;
-
   if (t>last_stream_time)
     last_stream_time=t;
 
 #ifdef DEBUG
   if (raw) {
+    int ret;
     if (op->packetno>0) {
       ogg_int64_t bytes=op->bytes;
       ret=fwrite(&bytes,1,8,f);
@@ -197,14 +208,9 @@ static const char *eat_arg(int argc,char **argv,int *n)
   return argv[++*n];
 }
 
-static kate_float hmsms2s(int h,int m,int s,int ms)
-{
-  return h*3600+m*60+s+ms/(kate_float)1000.0;
-}
-
 static kate_int64_t hmsms2ms(int h,int m,int s,int ms)
 {
-  return h*3600000+m*60000+s*1000+ms;
+  return h*(kate_int64_t)3600000+m*60000+s*1000+ms;
 }
 
 static char *fgets2(char *s,size_t len,FILE *f,int ignore_hash)
@@ -218,6 +224,7 @@ static char *fgets2(char *s,size_t len,FILE *f,int ignore_hash)
       if (ptr) strcpy(ptr,"\n");
     }
   }
+  else *s=0;
   return ret;
 }
 
@@ -242,29 +249,31 @@ static int is_line_empty(const char *s)
   return 1;
 }
 
-void update_stream_time(kate_state *k,FILE *fout,kate_float endt)
+void update_stream_time(kate_state *k,FILE *fout,kate_int64_t endt)
 {
   ogg_packet op;
-  kate_float t;
+  kate_int64_t t;
   int ret;
-  kate_float step;
+  kate_int64_t step;
+  kate_int64_t raw_repeat_threshold=kate_duration_granule(k->ki,repeat_threshold);
+  kate_int64_t raw_keepalive_threshold=kate_duration_granule(k->ki,keepalive_threshold);
 
   /* work out the best step to walk the time segment */
-  step=(kate_float)0;
-  if (step==(kate_float)0 || (repeat_threshold>(kate_float)0 && repeat_threshold<step))
-    step=repeat_threshold;
-  if (step==(kate_float)0 || (keepalive_threshold>(kate_float)0 && keepalive_threshold<step))
-    step=keepalive_threshold;
+  step=0;
+  if (step==0 || (raw_repeat_threshold>0 && raw_repeat_threshold<step))
+    step=raw_repeat_threshold;
+  if (step==0 || (raw_keepalive_threshold>0 && raw_keepalive_threshold<step))
+    step=raw_keepalive_threshold;
 
-  if (step<=(kate_float)0) return;
+  if (step<=0) return;
 
   for (t=last_stream_time;t<endt;t+=step) {
     /* try repeats first, no keepalives will be needed if we have one */
-    if (repeat_threshold>(kate_float)0) {
+    if (raw_repeat_threshold>0) {
       while (1) {
-        ret=kate_ogg_encode_repeat(k,t,repeat_threshold,&op);
+        ret=kate_ogg_encode_repeat_raw_times(k,t,raw_repeat_threshold,&op);
         if (ret<0) {
-          fprintf(stderr,"Failed encoding repeat at %f (%d), continuing anyway\n",t,ret);
+          fprintf(stderr,"Failed encoding repeat at %"PRId64" (%d), continuing anyway\n",t,ret);
           return;
         }
         if (ret==0) break;
@@ -276,10 +285,10 @@ void update_stream_time(kate_state *k,FILE *fout,kate_float endt)
       }
     }
     /* keepalives next */
-    if (keepalive_threshold>(kate_float)0 && t-last_stream_time>=keepalive_threshold) {
-      ret=kate_ogg_encode_keepalive(k,t,&op);
+    if (raw_keepalive_threshold>0 && t-last_stream_time>=raw_keepalive_threshold) {
+      ret=kate_ogg_encode_keepalive_raw_times(k,t,&op);
       if (ret<0) {
-        fprintf(stderr,"Failed encoding keepalive at %f (%d), continuing anyway\n",t,ret);
+        fprintf(stderr,"Failed encoding keepalive at %"PRId64" (%d), continuing anyway\n",t,ret);
         return;
       }
       ret=send_packet(fout,&op,t);
@@ -299,7 +308,7 @@ static void remove_last_newline(char *text)
   if (len>0 && text[len-1]=='\n') text[--len]=0;
 }
 
-static int emit_srt_event(FILE *fout,kate_int64_t t0,kate_int64_t t1,const char *text,int allow_srt_markup)
+static int emit_srt_event(FILE *fout,kate_int64_t t0,kate_int64_t t1,const char *text,int allow_srt_markup,int x1,int y1,int x2,int y2)
 {
   ogg_packet op;
   size_t len;
@@ -319,6 +328,20 @@ static int emit_srt_event(FILE *fout,kate_int64_t t0,kate_int64_t t1,const char
     return ret;
   }
   if (allow_srt_markup) kate_encode_set_markup_type(&k,kate_markup_simple);
+  if (x1!=-INT_MAX && y1!=-INT_MAX && x2!=-INT_MAX && y2!=-INT_MAX) {
+    kate_region kr;
+    kate_region_init(&kr);
+    kr.metric=kate_pixel;
+    kr.x=x1;
+    kr.y=y1;
+    kr.w=x2-x1+1;
+    kr.h=y2-y1+1;
+    ret=kate_encode_set_region(&k,&kr);
+    if (ret<0) {
+      fprintf(stderr,"Error setting region: %d\n",ret);
+      return ret;
+    }
+  }
   ret=kate_ogg_encode_text_raw_times(&k,t0,t1,text,len,&op);
   if (ret<0) {
     fprintf(stderr,"Error encoding text: %d\n",ret);
@@ -340,6 +363,7 @@ static int convert_srt(FILE *fin,FILE *fout,int allow_srt_markup)
   int id;
   static char text[4096];
   int h0,m0,s0,ms0,h1,m1,s1,ms1;
+  int x1,y1,x2,y2;
   kate_int64_t t0=0,t1=0;
   int line=0;
 
@@ -366,7 +390,7 @@ static int convert_srt(FILE *fin,FILE *fout,int allow_srt_markup)
     memmove(str,str+3,strlen(str+3)+1);
   }
 
-  while (!feof(fin)) {
+  while (!feof(fin) || *str) {
     ++line;
     switch (need) {
       case need_id:
@@ -387,25 +411,36 @@ static int convert_srt(FILE *fin,FILE *fout,int allow_srt_markup)
         strcpy(text,"");
         break;
       case need_timing:
-        ret=sscanf(str,"%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d\n",&h0,&m0,&s0,&ms0,&h1,&m1,&s1,&ms1);
-        if (ret!=8 || (h0|m0|s0|ms0)<0 || (h1|m1|s1|ms1)<0) {
-          fprintf(stderr,"Syntax error at line %d: %s\n",line,str);
-          return -1;
-        }
-        else {
-          t0=hmsms2ms(h0,m0,s0,ms0);
-          t1=hmsms2ms(h1,m1,s1,ms1);
-          if (t1<t0) {
-            fprintf(stderr,"Error at line %d: end time must not be less than start time\n",line);
+        ret=sscanf(str,"%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d %*[xX]1: %d %*[xX]2: %d %*[yY]1: %d %*[yY]2: %d\n",&h0,&m0,&s0,&ms0,&h1,&m1,&s1,&ms1,&x1,&x2,&y1,&y2);
+        if (ret!=12) {
+          x1=y1=x2=y2=-INT_MAX;
+          ret=sscanf(str,"%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d\n",&h0,&m0,&s0,&ms0,&h1,&m1,&s1,&ms1);
+          if (ret!=8) {
+            fprintf(stderr,"Syntax error at line %d: %s\n",line,str);
             return -1;
           }
         }
+
+        if ((h0|m0|s0|ms0)<0 || (h1|m1|s1|ms1)<0) {
+          fprintf(stderr,"Bad time specification at line %d: %s\n",line,str);
+          return -1;
+        }
+        t0=hmsms2ms(h0,m0,s0,ms0);
+        t1=hmsms2ms(h1,m1,s1,ms1);
+        if (t1<t0) {
+          fprintf(stderr,"Error at line %d: end time must not be less than start time\n",line);
+          return -1;
+        }
+        if (x2<x1 || y2<y1) {
+          fprintf(stderr,"Invalid region coordinates at line %d\n",line);
+          return -1;
+        }
         need=need_text;
         break;
       case need_text:
         if (is_line_empty(str)) {
           remove_last_newline(text);
-          ret=emit_srt_event(fout,t0,t1,text,allow_srt_markup);
+          ret=emit_srt_event(fout,t0,t1,text,allow_srt_markup,x1,y1,x2,y2);
           if (ret<0) return ret;
           need=need_id;
         }
@@ -423,17 +458,17 @@ static int convert_srt(FILE *fin,FILE *fout,int allow_srt_markup)
   if (need==need_text && text[0]) {
     fprintf(stderr, "Warning: last event is not followed by an empty line - input might be truncated\n");
     remove_last_newline(text);
-    ret=emit_srt_event(fout,t0,t1,text,allow_srt_markup);
+    ret=emit_srt_event(fout,t0,t1,text,allow_srt_markup,x1,y1,x2,y2);
     if (ret<0) return ret;
   }
 
   return 0;
 }
 
-static void add_kate_karaoke_tag(kate_motion *km,kate_float dt,const char *str,size_t len,int line)
+static void add_kate_karaoke_tag(kate_motion *km,kate_int64_t dt,const char *str,size_t len,int line)
 {
   kate_curve *kc;
-  kate_float ptr=(kate_float)-0.5;
+  kate_int64_t ptr=-500; /* minus half a second, in milliseconds */
   int ret;
   kate_curve **new_curves;
   kate_float *new_durations;
@@ -450,7 +485,7 @@ static void add_kate_karaoke_tag(kate_motion *km,kate_float dt,const char *str,s
       fprintf(stderr, "Error: line %d: failed to get UTF-8 glyph from string\n",line);
       return;
     }
-    ptr+=(kate_float)1.0;
+    ptr+=1000; /* a second, in milliseconds */
   }
   /* ptr now points to the middle of the glyph we're at */
 
@@ -461,7 +496,7 @@ static void add_kate_karaoke_tag(kate_motion *km,kate_float dt,const char *str,s
     kc->npts=1;
     kc->pts=(kate_float*)kate_malloc(2*sizeof(kate_float));
     if (kc->pts) {
-      kc->pts[0]=ptr;
+      kc->pts[0]=ptr/(kate_float)1000;
       kc->pts[1]=(kate_float)0;
 
       new_curves=(kate_curve**)kate_realloc(km->curves,(km->ncurves+1)*sizeof(kate_curve*));
@@ -472,7 +507,7 @@ static void add_kate_karaoke_tag(kate_motion *km,kate_float dt,const char *str,s
       if (new_curves && new_durations) {
         km->ncurves++;
         km->curves[km->ncurves-1]=kc;
-        km->durations[km->ncurves-1]=dt;
+        km->durations[km->ncurves-1]=dt/(kate_float)1000;
       }
       else {
         fprintf(stderr,"Error: failed to allocate memory to store curve/duration");
@@ -503,13 +538,13 @@ static int fraction_to_milliseconds(int fraction,int digits)
   return fraction;
 }
 
-static kate_motion *process_enhanced_lrc_tags(char *str,kate_float start_time,kate_float end_time,int line)
+static kate_motion *process_enhanced_lrc_tags(char *str,kate_int64_t start_time,kate_int64_t end_time,int line)
 {
   char *start,*end;
   int ret;
   int m,s,fs;
   kate_motion *km=NULL;
-  kate_float current_time=start_time;
+  kate_int64_t current_time=start_time;
   int f0,f1;
 
   if (!str) return NULL;
@@ -532,7 +567,7 @@ static kate_motion *process_enhanced_lrc_tags(char *str,kate_float start_time,ka
       fprintf(stderr,"Error: failed to process enhanced LRC tag (%*.*s) - ignored\n",(int)(end-start+1),(int)(end-start+1),start);
     }
     else {
-      kate_float tag_time=hmsms2s(0,m,s,fraction_to_milliseconds(fs,f1-f0));
+      kate_int64_t tag_time=hmsms2ms(0,m,s,fraction_to_milliseconds(fs,f1-f0));
 
       /* if this is the first tag in this line, create a kate motion */
       if (!km) {
@@ -584,13 +619,14 @@ static void add_kate_karaoke_style(kate_info *ki,unsigned char r,unsigned char g
       fprintf(stderr,"WARNING - failed to allocate memory for Kate karaoke style\n");
     }
 }
+
 static int convert_lrc(FILE *fin,FILE *fout)
 {
   int ret;
   static char text[4096];
   int m,s,fs;
-  kate_float t;
-  kate_float start_time=(kate_float)-1.0;
+  kate_int64_t t;
+  kate_int64_t start_time=-1;
   int offset;
   int line=0;
   fpos_t start;
@@ -666,7 +702,7 @@ static int convert_lrc(FILE *fin,FILE *fout)
   ret=flush_headers(fout);
   if (ret<0) return ret;
 
-  while (!feof(fin)) {
+  while (!feof(fin) || *str) {
     if (!is_line_empty(str)) {
       f0=f1=-1;
       ret=sscanf(str,"[%d:%d.%n%d%n]%n",&m,&s,&f0,&fs,&f1,&offset);
@@ -674,8 +710,8 @@ static int convert_lrc(FILE *fin,FILE *fout)
         fprintf(stderr,"Syntax error at line %d: %s\n",line,str);
         return -1;
       }
-      t=hmsms2s(0,m,s,fraction_to_milliseconds(fs,f1-f0));
-      if (start_time>=0.0 && !is_line_empty(text)) {
+      t=hmsms2ms(0,m,s,fraction_to_milliseconds(fs,f1-f0));
+      if (start_time>=0 && !is_line_empty(text)) {
         ogg_packet op;
 
         update_stream_time(&k,fout,start_time);
@@ -708,7 +744,7 @@ static int convert_lrc(FILE *fin,FILE *fout)
           }
           return ret;
         }
-        ret=kate_ogg_encode_text(&k,start_time,t,text,len,&op);
+        ret=kate_ogg_encode_text_raw_times(&k,start_time,t,text,len,&op);
         if (ret<0) {
           fprintf(stderr,"Failed to add lyrics (%d)\n",ret);
           return ret;
@@ -782,7 +818,7 @@ static void set_encoder_comment(kate_comment *kc)
   size_t len=strlen(base)+strlen(version)+1;
   char *value=(char*)kate_malloc(len);
   if (!value) {
-    fprintf(stderr,"Failed to allocate %zu bytes\n",len);
+    fprintf(stderr,"Failed to allocate %lu bytes\n",(unsigned long)len);
     exit(-1);
   }
   sprintf(value,"%s%s",base,version);
@@ -1035,7 +1071,7 @@ int main(int argc,char **argv)
       failed=ret;
     }
     else {
-      ret=send_packet(fout,&op,(kate_float)-1);
+      ret=send_packet(fout,&op,-1);
       if (ret<0) {
         fprintf(stderr,"error sending end packet: %d\n",ret);
         failed=ret;
