@@ -63,14 +63,26 @@ static void write_text(FILE *f,const char *text,size_t len0,kate_markup_type tex
     }
     else {
       /* be conservative in what we encode */
-      const char *escape="";
+      const char *escape_list="";
       switch (text_markup_type) {
-        case kate_markup_none: escape="\"\r\n`'|\\"; break;
-        case kate_markup_simple: escape="\"\r\n`'|\\"; break;
+        case kate_markup_none: escape_list="\"\r\n`'|\\"; break;
+        case kate_markup_simple: escape_list="\"\r\n`'|\\"; break;
         default: fprintf(stderr,"Unknown text markup type (%d)\n",text_markup_type); break;
       }
-      if (ret>=32 && ret<127 && !strchr(escape,ret)) {
-        fprintf(f,"%c",ret);
+      if (ret>=32 && (ret>0xff || !strchr(escape_list,ret))) {
+        char utf8[12],*utf8ptr=utf8;
+        size_t wlen0=sizeof(utf8);
+        ret=kate_text_set_character(kate_utf8,ret,&utf8ptr,&wlen0);
+        if (ret<0) {
+          fprintf(stderr,"Error writing character\n");
+          break;
+        }
+        ret=kate_text_set_character(kate_utf8,0,&utf8ptr,&wlen0);
+        if (ret<0) {
+          fprintf(stderr,"Error writing character\n");
+          break;
+        }
+        fprintf(f,"%s",utf8);
       }
       else {
         fprintf(f,"&#%u;",ret);
@@ -91,6 +103,64 @@ static void write_color(FILE *f,const char *name,const kate_color *kc,size_t ind
   kate_free(sindent);
 }
 
+static int is_suitable_string(const char *value,size_t len)
+{
+  const char *p;
+  int ret;
+  size_t n;
+  static const char *list="\"\r\n";
+
+  ret=kate_text_validate(kate_utf8,value,len);
+  if (ret<0) return 0;
+  if (len>0) for (n=0;n<len-1;++n) if (!value[n]) return 0;
+  for (p=list;*p;++p) if (memchr(value,*p,len)) return 0;
+  return 1;
+}
+
+static void write_metadata(FILE *f,const kate_meta *km,size_t indent)
+{
+  char *sindent=(char*)kate_malloc(1+indent);
+  size_t count,n,b;
+  int ret;
+
+  for (n=0;n<indent;++n) sindent[n]=' ';
+  sindent[indent]=0;
+
+  ret=kate_meta_query_count(km);
+  if (ret<0) {
+    fprintf(stderr,"Error retrieving medata: %d\n",ret);
+  }
+  else {
+    count=ret;
+    for (n=0;n<count;++n) {
+      const char *tag,*value;
+      size_t len;
+      ret=kate_meta_query(km,n,&tag,&value,&len);
+      if (ret<0) {
+        fprintf(stderr,"Error retrieving medata: %d\n",ret);
+      }
+      else {
+        fprintf(f,"%smeta \"%s\"=",sindent,tag);
+        if (is_suitable_string(value,len)) {
+          fprintf(f,"\"%s\"",value);
+        }
+        else {
+          fprintf(f," %lu {\n",(unsigned long)len);
+          for (b=0;b<len;++b) {
+            if (b%16==0) fprintf(f,"%s  ",sindent);
+            fprintf(f," 0x%02x",value[b]);
+            if (b%16==15 || b==len-1) fprintf(f,"\n");
+          }
+          fprintf(f,"%s}",sindent);
+        }
+        fprintf(f,"\n");
+      }
+    }
+  }
+
+  kate_free(sindent);
+}
+
 static void write_style_defs(FILE *f,const kate_style *ks,size_t indent)
 {
   char *sindent=(char*)kate_malloc(1+indent);
@@ -139,6 +209,7 @@ static void write_style_defs(FILE *f,const kate_style *ks,size_t indent)
     if (ks->strike) fprintf(f,"%sstrike\n",sindent);
     if (ks->justify) fprintf(f,"%sjustify\n",sindent);
     fprintf(f,"%swrap %s\n",sindent,wrap2text(ks->wrap_mode));
+    if (ks->meta) write_metadata(f,ks->meta,indent);
   }
 
   kate_free(sindent);
@@ -154,6 +225,7 @@ static void write_region_defs(FILE *f,const kate_region *kr,size_t indent)
   fprintf(f,"%s%s position %d %d size %d %d\n",sindent,metric2text(kr->metric),kr->x,kr->y,kr->w,kr->h);
   if (kr->style>=0) fprintf(f,"%sdefault style %d\n",sindent,kr->style);
   if (kr->clip) fprintf(f,"%sclip\n",sindent);
+  if (kr->meta) write_metadata(f,kr->meta,indent);
 
   kate_free(sindent);
 }
@@ -168,7 +240,7 @@ static void write_curve_defs(FILE *f,const kate_curve *kc,size_t indent)
   fprintf(f,"%s%s\n",sindent,curve2text(kc->type));
   if (kc->npts>0) {
     size_t pt;
-    fprintf(f,"%s%zu points {\n",sindent,kc->npts);
+    fprintf(f,"%s%lu points {\n",sindent,(unsigned long)kc->npts);
     for (pt=0;pt<kc->npts;++pt) fprintf(f,"%s  %.16g %.16g\n",sindent,kc->pts[pt*2],kc->pts[pt*2+1]);
     fprintf(f,"%s}\n",sindent);
   }
@@ -215,6 +287,7 @@ static void write_motion_defs(FILE *f,const kate_info *ki,const kate_motion *km,
       fprintf(f,"\n");
     }
   }
+  if (km->meta) write_metadata(f,km->meta,indent);
 
   kate_free(sindent);
 }
@@ -227,12 +300,13 @@ static void write_palette_defs(FILE *f,const kate_palette *kp,size_t indent)
   for (n=0;n<indent;++n) sindent[n]=' ';
   sindent[indent]=0;
 
-  fprintf(f,"%s%zu colors {\n",sindent,kp->ncolors);
+  fprintf(f,"%s%lu colors {\n",sindent,(unsigned long)kp->ncolors);
   for (s=0;s<kp->ncolors;++s) {
     const kate_color *kc=kp->colors+s;
     fprintf(f,"%s  { %d %d %d %d },\n",sindent,kc->r,kc->g,kc->b,kc->a);
   }
   fprintf(f,"%s}\n",sindent);
+  if (kp->meta) write_metadata(f,kp->meta,indent);
 
   kate_free(sindent);
 }
@@ -250,7 +324,7 @@ static void write_paletted_bitmap(const char *filename,const kate_bitmap *kb,con
     return;
   }
 
-  fprintf(f,"P6\n%zu %zu\n255\n",kb->width,kb->height);
+  fprintf(f,"P6\n%lu %lu\n255\n",(unsigned long)kb->width,(unsigned long)kb->height);
   n=0;
   for (y=0;y<kb->height;++y) {
     for (x=0;x<kb->width;++x) {
@@ -288,16 +362,16 @@ static void write_bitmap_defs(FILE *f,const kate_bitmap *kb,size_t indent)
 
   switch (kb->type) {
     case kate_bitmap_type_png:
-      fprintf(f,"%s%zux%zu png %zu {\n",sindent,kb->width,kb->height,kb->size);
+      fprintf(f,"%s%lux%lu png %lu {\n",sindent,(unsigned long)kb->width,(unsigned long)kb->height,(unsigned long)kb->size);
       for (p=0;p<kb->size;++p) {
         if (p%16==0) fprintf(f,"%s",sindent);
-        fprintf(f," %3d",kb->pixels[p]);
+        fprintf(f," 0x%02x",kb->pixels[p]);
         if (p%16==15 || p==kb->size-1) fprintf(f,"\n");
       }
       fprintf(f,"%s}\n",sindent);
       break;
     case kate_bitmap_type_paletted:
-      fprintf(f,"%s%zux%zux%d {\n",sindent,kb->width,kb->height,kb->bpp);
+      fprintf(f,"%s%lux%lux%d {\n",sindent,(unsigned long)kb->width,(unsigned long)kb->height,kb->bpp);
       p=0;
       for (h=0;h<kb->height;++h) {
         fprintf(f,"%s ",sindent);
@@ -316,6 +390,7 @@ static void write_bitmap_defs(FILE *f,const kate_bitmap *kb,size_t indent)
     fprintf(f,"%soffset %d %d\n",sindent,kb->x_offset,kb->y_offset);
   }
   if (kb->palette>=0) fprintf(f,"%sdefault palette %d\n",sindent,kb->palette);
+  if (kb->meta) write_metadata(f,kb->meta,indent);
 
   kate_free(sindent);
 }
@@ -380,7 +455,7 @@ static void write_canvas_size(FILE *f,const kate_info *ki,size_t indent)
   sindent[indent]=0;
 
   if (ki->original_canvas_width>0 || ki->original_canvas_height>0) {
-    fprintf(f,"%scanvas size %zu %zu\n",sindent,ki->original_canvas_width,ki->original_canvas_height);
+    fprintf(f,"%scanvas size %lu %lu\n",sindent,(unsigned long)ki->original_canvas_width,(unsigned long)ki->original_canvas_height);
   }
 
   kate_free(sindent);
@@ -650,6 +725,7 @@ void write_kate_event(FILE *fout,void *data,const kate_event *ev,ogg_int64_t gra
       }
     }
   }
+  if (ev->meta) write_metadata(fout,ev->meta,4);
   fprintf(fout,"  }\n");
   fprintf(fout,"\n");
 }
